"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.print = exports.indentify = exports.severityStatusIcon = exports.outputManifest = exports.produceSourceLine = exports.outputSource = exports.outputLintSummary = exports.outputLintResult = exports.output = void 0;
const chalk_1 = __importDefault(require("chalk"));
const node_emoji_1 = __importDefault(require("node-emoji"));
function output(result, params) {
    var _a, _b;
    params = params !== null && params !== void 0 ? params : {};
    params.skipSummary = (_a = params.skipSummary) !== null && _a !== void 0 ? _a : false;
    params.skipResult = (_b = params.skipResult) !== null && _b !== void 0 ? _b : false;
    if (!result.foundK8sVersion) {
        print(`${node_emoji_1.default.get('x')}  Failed to find Kubernetes Version ${result.targetK8sVersion}`);
    }
    else {
        if (!result.foundExactK8sVersion) {
            print(`${node_emoji_1.default.get('warning')}  Could not find requested Kubernetes Version: ${result.targetK8sVersion}`);
        }
    }
    print(`${node_emoji_1.default.get('information_source')}  Linting against Kubernetes Version: ${result.selectedK8sVersion}`);
    print();
    for (const source of result.sources) {
        outputSource(source);
        outputErrors(source, 3);
        for (const manifest of source.manifests) {
            outputManifest(manifest, objectSeverityIcon(manifest), 3);
            outputErrors(manifest, 6);
        }
        print();
    }
    print();
    if (!params.skipSummary) {
        outputLintSummary(result);
        print();
    }
    if (!params.skipResult) {
        outputLintResult(result);
    }
}
exports.output = output;
function outputLintResult(result) {
    if (result.success) {
        print(`${node_emoji_1.default.get('white_check_mark')} Lint Succeeded.`);
    }
    else {
        print(`${node_emoji_1.default.get('x')} Lint Failed`);
    }
}
exports.outputLintResult = outputLintResult;
function outputLintSummary(result) {
    print(chalk_1.default.underline('Lint Summary'));
    print(`Sources: ${result.counters.sources.total}`, 4);
    print(`${severityStatusIcon('fail')} Sources with Errors: ${result.counters.sources.withErrors}`, 8);
    print(`Manifests: ${result.counters.manifests.total}`, 4);
    print(`${severityStatusIcon('pass')} Manifests Passed: ${result.counters.manifests.passed}`, 8);
    print(`${severityStatusIcon('fail')} Manifests with Errors: ${result.counters.manifests.withErrors}`, 8);
    print(`${severityStatusIcon('warning')} Manifests with Warnings: ${result.counters.manifests.withWarnings}`, 8);
}
exports.outputLintSummary = outputLintSummary;
function outputSource(source, indent) {
    const parts = [];
    parts.push(objectSeverityIcon(source));
    parts.push(produceSourceLine(source));
    print(parts.join(' '), indent);
}
exports.outputSource = outputSource;
function produceSourceLine(source) {
    const parts = [];
    if (source.kind === 'file') {
        parts.push(node_emoji_1.default.get('page_facing_up'));
    }
    else if (source.kind === 'web') {
        parts.push(node_emoji_1.default.get('globe_with_meridians'));
    }
    else if (source.kind === 'stream') {
        parts.push(node_emoji_1.default.get('aquarius'));
    }
    else if (source.kind === 'k8s') {
        parts.push('☸️ ');
    }
    parts.push(`${source.kind.toUpperCase()}:`);
    parts.push(source.path);
    return parts.join(' ');
}
exports.produceSourceLine = produceSourceLine;
function outputManifest(manifest, icon, indent) {
    const namingParts = [];
    if (manifest.namespace) {
        namingParts.push(`Namespace: ${manifest.namespace}`);
    }
    namingParts.push(`API: ${manifest.apiVersion}`);
    namingParts.push(`Kind: ${manifest.kind}`);
    namingParts.push(`Name: ${manifest.name}`);
    const parts = [];
    parts.push(icon);
    parts.push(namingParts.join(', '));
    print(parts.join(' '), indent);
}
exports.outputManifest = outputManifest;
function outputErrors(obj, indent) {
    if (!obj.success) {
        if (obj.errors) {
            for (const msg of obj.errors) {
                const line = `${severityStatusIcon('fail')} ${msg}`;
                print(line, indent);
            }
        }
    }
    if (obj.warnings) {
        for (const msg of obj.warnings) {
            const line = `${severityStatusIcon('warning')} ${msg}`;
            print(line, indent);
        }
    }
}
function objectSeverityIcon(obj) {
    return severityStatusIcon(obj.severity);
}
function severityStatusIcon(severity) {
    let iconName = 'question';
    if (severity === 'pass') {
        iconName = 'white_check_mark';
    }
    else if (severity === 'fail') {
        iconName = 'x';
    }
    else if (severity === 'warning') {
        return `${node_emoji_1.default.get('warning')} `;
    }
    return `${node_emoji_1.default.get(iconName)}`;
}
exports.severityStatusIcon = severityStatusIcon;
function indentify(str, count) {
    const prefix = ' '.repeat(count !== null && count !== void 0 ? count : 0);
    let lines = str.split('\n');
    lines = lines.map(x => `${prefix}${x}`);
    return lines.join('\n');
}
exports.indentify = indentify;
function print(str, indent) {
    console.log(indentify(str !== null && str !== void 0 ? str : "", indent));
}
exports.print = print;
//# sourceMappingURL=output.js.map