"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoteK8sRegistry = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const k8s_1 = require("../utils/k8s");
const client_side_filtering_1 = require("./client-side-filtering");
const k8s_manifest_1 = require("../manifests/k8s-manifest");
class RemoteK8sRegistry {
    constructor(logger, k8sConnector) {
        this._source = {
            source: {
                kind: "k8s",
                path: "live"
            },
            contents: [],
            success: true,
            errors: [],
            warnings: [],
        };
        this._logger = logger.sublogger('RemoteK8sRegistry');
        this._k8sConnector = k8sConnector;
        this._client = k8sConnector.client;
    }
    query(query) {
        var _a;
        this._logger.info("[query] ", query);
        let apiName = undefined;
        let version = undefined;
        if (query.isApiVersion) {
            const apiVersion = (0, k8s_1.parseApiVersion)(query.apiVersion);
            apiName = (apiVersion === null || apiVersion === void 0 ? void 0 : apiVersion.group) || null;
            version = apiVersion === null || apiVersion === void 0 ? void 0 : apiVersion.version;
        }
        else {
            if (query.apiOrNone) {
                apiName = query.apiOrNone;
            }
            else {
                apiName = null;
            }
            if (query.version) {
                version = query.version;
            }
        }
        this._logger.info("[query] apiName: %s", apiName);
        this._logger.info("[query] version: %s", version);
        if (!query.kind) {
            this._logger.info("[query] No Kind");
            return [];
        }
        const resourceClient = this._client.client(query.kind, apiName, version);
        if (!resourceClient) {
            this._logger.info("[query] Unknown Resource");
            return [];
        }
        let results = [];
        if (query.nameFilters && query.nameFilters.length > 0) {
            const namespace = (_a = query.namespace) !== null && _a !== void 0 ? _a : null;
            const nameDict = the_lodash_1.default.makeBoolDict(query.nameFilters);
            for (const name of the_lodash_1.default.keys(nameDict)) {
                const item = resourceClient.querySync(namespace, name);
                if (item) {
                    results.push(item);
                }
            }
            results = results.filter(x => x.id.name && nameDict[x.id.name]);
        }
        else {
            // TODO: Try using label filter as an optimization:.
            results = resourceClient.queryAllSync(query.namespace);
        }
        this._logger.info("[query] result count: %s", results.length);
        const manifests = results.map(x => this._makeManifest(x));
        const filtering = new client_side_filtering_1.ClientSideFiltering(manifests);
        filtering.applyLabelFilter(query.labelFilters);
        return filtering.items;
    }
    _makeManifest(config) {
        const k8sObject = config;
        const k8sManifest = new k8s_manifest_1.K8sManifest(k8sObject, this._source);
        return k8sManifest;
    }
}
exports.RemoteK8sRegistry = RemoteK8sRegistry;
//# sourceMappingURL=remote-k8s-registry.js.map