"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleRuntime = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
class RuleRuntime {
    constructor(logger, manifest, ruleObject, ruleEngineReporter, compiler, application, values) {
        this._violations = [];
        this._passed = [];
        this._logger = logger.sublogger('RuleObject');
        this._manifest = manifest;
        this._ruleObject = ruleObject;
        this._ruleEngineReporter = ruleEngineReporter;
        this._compiler = compiler;
        this._application = application;
        this._values = values;
    }
    get rule() {
        return this._compiler.rule;
    }
    get isCompiled() {
        return this._compiler.isCompiled;
    }
    get hasRuntimeErrors() {
        return this._compiler.hasRuntimeErrors;
    }
    get ruleErrors() {
        return this._compiler.ruleErrors;
    }
    get violations() {
        return this._violations;
    }
    get passed() {
        return this._passed;
    }
    execute() {
        this._logger.info('[execute] %s :: %s :: %s...', this._ruleObject.kind, this._ruleObject.namespace, this._ruleObject.name);
        return the_promise_1.Promise.resolve()
            .then(() => {
            if (this._compiler.isCompiled) {
                return this._processTarget()
                    .then(results => {
                    this._logger.info('[execute] Targets Count: %s', results.length);
                    return the_promise_1.Promise.serial(results, x => this._processValidation(x));
                });
            }
        })
            .then(() => {
            var _a;
            if (!this.isCompiled || this.hasRuntimeErrors) {
                if (!((_a = this._manifest.errorsWithRule) !== null && _a !== void 0 ? _a : false)) {
                    this._manifest.errorsWithRule = true;
                    this._ruleEngineReporter.manifestPackage.manifestError(this._manifest, "Failed to compile the rule.");
                }
            }
        });
    }
    _processTarget() {
        return the_promise_1.Promise.resolve()
            .then(() => {
            return this._compiler.targetProcessor.execute(this._application, this._values);
        })
            .catch(reason => {
            this._logger.error("Failed to execute the rule %s", this._ruleObject.name, reason);
            this._compiler.reportScriptErrors('rule', reason.message);
            return [];
        });
    }
    _processValidation(item) {
        this._logger.info("[_processValidation] %s :: %s", item.apiVersion, item.kind);
        return the_promise_1.Promise.resolve()
            .then(() => {
            return this._compiler.validationProcessor.execute(item, this._values)
                .then(result => {
                this._logger.info("[_processValidation]  result: ", result);
                if (result.success) {
                    this._processValidationResult(item.manifest, result);
                }
                else {
                    if (result.messages && (result.messages.length > 0)) {
                        this._compiler.reportScriptErrors('rule', result.messages);
                    }
                    else {
                        this._compiler.reportScriptErrors('rule', ['Unknown error executing the rule.']);
                    }
                }
            });
        });
    }
    _processValidationResult(manifest, result) {
        manifest.rules.processed = true;
        if ((the_lodash_1.default.keys(result.validation.errorMsgs).length + the_lodash_1.default.keys(result.validation.warnMsgs).length) > 0) {
            if (the_lodash_1.default.keys(result.validation.errorMsgs).length > 0) {
                manifest.rules.errors = true;
            }
            if (the_lodash_1.default.keys(result.validation.warnMsgs).length > 0) {
                manifest.rules.warnings = true;
            }
            const manifestViolations = {
                manifest: manifest,
                hasErrors: (the_lodash_1.default.keys(result.validation.errorMsgs).length > 0),
                hasWarnings: (the_lodash_1.default.keys(result.validation.warnMsgs).length > 0),
            };
            for (const error of the_lodash_1.default.keys(result.validation.errorMsgs)) {
                this._reportError(manifest, error);
                if (!manifestViolations.errors) {
                    manifestViolations.errors = [];
                }
                manifestViolations.errors.push(error);
            }
            for (const warn of the_lodash_1.default.keys(result.validation.warnMsgs)) {
                this._reportWarning(manifest, warn);
                if (!manifestViolations.warnings) {
                    manifestViolations.warnings = [];
                }
                manifestViolations.warnings.push(warn);
            }
            this._violations.push(manifestViolations);
        }
        else {
            this._passed.push(manifest);
        }
    }
    _reportError(manifest, msg) {
        const manifestMsg = `Rule ${this._ruleObject.name} violated. ${msg}.`;
        this._ruleEngineReporter.reportError(this._ruleObject, manifest, manifestMsg);
    }
    _reportWarning(manifest, msg) {
        const manifestMsg = `Rule ${this._ruleObject.name} warned. ${msg}.`;
        this._ruleEngineReporter.reportWarning(this._ruleObject, manifest, manifestMsg);
    }
}
exports.RuleRuntime = RuleRuntime;
//# sourceMappingURL=rule-runtime.js.map