"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RulesRuntime = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const rule_runtime_1 = require("./rule-runtime");
const execution_context_1 = require("./execution-context");
const rule_engine_reporter_1 = require("../reporting/rule-engine-reporter");
const spinner_1 = require("../../screen/spinner");
const rule_compiler_1 = require("../compiler/rule-compiler");
class RulesRuntime {
    constructor(logger, ruleRegistry, manifestPackage, namespaces, targetQueryExecutor, validatorQueryExecutor) {
        this._clusterRules = {};
        this._rules = [];
        this._logger = logger.sublogger('RulesRuntime');
        this._ruleRegistry = ruleRegistry;
        this._namespaces = namespaces;
        this._targetExecutionContext = new execution_context_1.ExecutionContext(this._logger, targetQueryExecutor, manifestPackage);
        this._validatorExecutionContext = new execution_context_1.ExecutionContext(this._logger, validatorQueryExecutor, manifestPackage);
        this._ruleEngineReporter = new rule_engine_reporter_1.RuleEngineReporter(this._logger, manifestPackage);
    }
    get rules() {
        return this._rules;
    }
    init() {
        return the_promise_1.Promise.resolve()
            .then(() => this._initClusterRules())
            .then(() => this._initRules())
            .then(() => this._initRuleApplicators());
    }
    execute() {
        this._spinner = (0, spinner_1.spinOperation)('Validating rules...');
        return the_promise_1.Promise.serial(this._rules, x => this._executeRule(x))
            .then(() => {
            this._spinner.complete('Rules validation complete.');
        });
    }
    _initClusterRules() {
        return the_promise_1.Promise.serial(this._ruleRegistry.clusterRules, x => {
            return this._initClusterRule(x);
        });
    }
    _initClusterRule(clusterRule) {
        this._logger.info("[_initClusterRule] %s...", clusterRule.name);
        if (clusterRule.spec.disabled) {
            this._clusterRules[clusterRule.name] = {
                rule: clusterRule
            };
            return;
        }
        const compiler = new rule_compiler_1.RuleCompiler(this._logger, clusterRule, this._targetExecutionContext, this._validatorExecutionContext);
        return compiler.compile()
            .then(() => {
            this._clusterRules[clusterRule.name] = {
                rule: clusterRule,
                compiler: compiler,
            };
            if (clusterRule.clustered) {
                this._setupClusterRuleRuntime(clusterRule, compiler);
            }
            else {
                if (!clusterRule.useApplicator) {
                    this._setupClusterRuleApplication(clusterRule, compiler);
                }
            }
        });
    }
    _setupClusterRuleApplication(clusterRule, compiler) {
        let namespaces = this._namespaces;
        if (clusterRule.onlySelectedNamespaces) {
            namespaces = namespaces.filter(x => clusterRule.namespaces[x]);
        }
        for (const ns of namespaces) {
            this._setupClusterRuleRuntime(clusterRule, compiler, ns);
        }
    }
    _setupClusterRuleRuntime(clusterRule, compiler, namespace) {
        this._logger.info("[_setupClusterRuleRuntime] >>> %s...", clusterRule.name);
        const application = {};
        if (namespace) {
            application.namespace = namespace;
        }
        let values = clusterRule.values;
        if (namespace) {
            const ruleNamespaceInfo = clusterRule.namespaces[namespace];
            values = this._makeValues([ruleNamespaceInfo === null || ruleNamespaceInfo === void 0 ? void 0 : ruleNamespaceInfo.values, values]);
        }
        const ruleRuntime = new rule_runtime_1.RuleRuntime(this._logger, clusterRule.manifest, compiler.rule, this._ruleEngineReporter, compiler, application, values);
        this._rules.push(ruleRuntime);
    }
    _initRules() {
        return the_promise_1.Promise.serial(this._ruleRegistry.rules, x => {
            return this._initRule(x);
        });
    }
    _initRule(rule) {
        this._logger.info("[_initRule] %s :: %s...", rule.namespace, rule.name);
        if (rule.spec.disabled) {
            return;
        }
        const compiler = new rule_compiler_1.RuleCompiler(this._logger, rule, this._targetExecutionContext, this._validatorExecutionContext);
        return compiler.compile()
            .then(() => {
            const ruleRuntime = new rule_runtime_1.RuleRuntime(this._logger, rule.manifest, rule, this._ruleEngineReporter, compiler, { namespace: rule.namespace }, rule.values);
            this._rules.push(ruleRuntime);
        });
    }
    _initRuleApplicators() {
        return the_promise_1.Promise.serial(this._ruleRegistry.ruleApplicators, x => {
            return this._initRuleApplicator(x);
        });
    }
    _initRuleApplicator(applicator) {
        this._logger.info("[_initRuleApplicator] %s :: %s...", applicator.namespace, applicator.name);
        if (applicator.spec.disabled) {
            return;
        }
        const clusterRefName = applicator.spec.clusterRuleRef.name;
        if (!clusterRefName) {
            return;
        }
        const clusterRuleInfo = this._clusterRules[clusterRefName];
        if (!clusterRuleInfo) {
            return;
        }
        if (!clusterRuleInfo.compiler) {
            return;
        }
        if (!clusterRuleInfo.rule.useApplicator) {
            return;
        }
        if (clusterRuleInfo.rule.spec.disabled) {
            return;
        }
        const ruleNamespaceInfo = clusterRuleInfo.rule.namespaces[applicator.namespace];
        if (clusterRuleInfo.rule.onlySelectedNamespaces) {
            if (!ruleNamespaceInfo) {
                return;
            }
        }
        const ruleConfig = the_lodash_1.default.clone(clusterRuleInfo.rule);
        ruleConfig.kind = applicator.kind;
        ruleConfig.name = applicator.name;
        ruleConfig.namespace = applicator.namespace;
        const ruleRuntime = new rule_runtime_1.RuleRuntime(this._logger, applicator.manifest, ruleConfig, this._ruleEngineReporter, clusterRuleInfo.compiler, { namespace: ruleConfig.namespace }, this._makeValues([applicator.values, ruleNamespaceInfo === null || ruleNamespaceInfo === void 0 ? void 0 : ruleNamespaceInfo.values, ruleConfig.values]));
        this._rules.push(ruleRuntime);
    }
    _executeRule(rule) {
        return the_promise_1.Promise.resolve()
            .then(() => rule.execute());
    }
    _makeValues(valuesArray) {
        valuesArray = valuesArray.map(x => x);
        let result = {};
        for (const x of valuesArray) {
            result = the_lodash_1.default.defaults(result, x);
        }
        return result;
    }
}
exports.RulesRuntime = RulesRuntime;
//# sourceMappingURL=rules-runtime.js.map