"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleRegistry = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const types_1 = require("./types");
const spinner_1 = require("../../screen/spinner");
const KUBEVIOUS_API_NAME = 'kubevious.io';
const KUBEVIOUS_KIND_CLUSTER_RULE = 'ClusterRule';
const KUBEVIOUS_KIND_RULE = 'Rule';
const KUBEVIOUS_KIND_APPLICATOR_RULE = 'RuleApplicator';
class RuleRegistry {
    constructor(logger) {
        this._clusterRules = {};
        this._rules = {};
        this._ruleApplicators = {};
        this._logger = logger.sublogger('RuleRegistry');
    }
    get clusterRules() {
        return the_lodash_1.default.values(this._clusterRules);
    }
    get rules() {
        return the_lodash_1.default.values(this._rules);
    }
    get ruleApplicators() {
        return the_lodash_1.default.values(this._ruleApplicators);
    }
    findClusterRule(name) {
        var _a;
        return (_a = this._clusterRules[name]) !== null && _a !== void 0 ? _a : null;
    }
    loadLocally(registry, options) {
        this._logger.info("[loadLocally] options: ", options);
        const spinner = (0, spinner_1.spinOperation)("Populating local RulesLibrary...");
        this._loadClusterRules(registry, options);
        this._loadNsRules(registry, options);
        this._loadApplicatorRules(registry, options);
        spinner.complete("RulesLibrary locally populated.");
    }
    loadRemotely(registry, options) {
        this._logger.info("[loadRemotely] options: ", options);
        const spinner = (0, spinner_1.spinOperation)("Populating RulesLibrary From K8s...");
        this._loadClusterRules(registry, options);
        this._loadNsRules(registry, options);
        this._loadApplicatorRules(registry, options);
        spinner.complete("RulesLibrary populated from K8s.");
    }
    _loadClusterRules(registry, options) {
        if (options.skipClusterScope) {
            return;
        }
        const spinner = (0, spinner_1.spinOperation)("Loading ClusterRules...", 2);
        const query = {
            isApiVersion: false,
            apiOrNone: KUBEVIOUS_API_NAME,
            kind: KUBEVIOUS_KIND_CLUSTER_RULE,
            isAllNamespaces: true,
        };
        const results = registry.query(query);
        for (const manifest of results) {
            this._loadClusterRule(manifest);
        }
        spinner.complete("ClusterRules loaded.");
    }
    _loadClusterRule(manifest) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const config = manifest.config;
        const name = (_a = config.metadata) === null || _a === void 0 ? void 0 : _a.name;
        if (!name) {
            return;
        }
        this._logger.info("[_loadClusterRule] %s...", name);
        const spec = (_b = config.spec) !== null && _b !== void 0 ? _b : {};
        spec.disabled = (_c = spec.disabled) !== null && _c !== void 0 ? _c : false;
        spec.values = (_d = spec.values) !== null && _d !== void 0 ? _d : {};
        spec.application = (_e = spec.application) !== null && _e !== void 0 ? _e : {};
        spec.application.clustered = (_f = spec.application.clustered) !== null && _f !== void 0 ? _f : false;
        spec.application.useApplicator = (_g = spec.application.useApplicator) !== null && _g !== void 0 ? _g : false;
        spec.application.onlySelectedNamespaces = (_h = spec.application.onlySelectedNamespaces) !== null && _h !== void 0 ? _h : false;
        spec.application.namespaces = (_j = spec.application.namespaces) !== null && _j !== void 0 ? _j : [];
        const clustered = spec.application.clustered;
        this._clusterRules[name] = {
            manifest: manifest,
            source: manifest.source.source,
            kind: types_1.RuleKind.ClusterRule,
            name: name,
            target: spec.target,
            script: spec.rule,
            values: spec.values,
            spec: spec,
            clustered: clustered,
            useApplicator: !clustered && spec.application.useApplicator,
            onlySelectedNamespaces: !clustered && spec.application.onlySelectedNamespaces,
            namespaces: clustered ? {} :
                the_lodash_1.default.makeDict(spec.application.namespaces, x => x.name, x => { var _a; return ({ values: (_a = x.values) !== null && _a !== void 0 ? _a : {} }); })
        };
    }
    _loadNsRules(registry, options) {
        this._logger.info("[_loadNsRules] ", options);
        const spinner = (0, spinner_1.spinOperation)("Loading Rules...", 2);
        if (options.onlySelectedNamespaces) {
            for (const namespace of options.namespaces) {
                spinner.update(`Loading Rules from ${namespace}...`);
                const query = {
                    isApiVersion: false,
                    apiOrNone: KUBEVIOUS_API_NAME,
                    kind: KUBEVIOUS_KIND_RULE,
                    namespace: namespace,
                };
                this._logger.info("[_loadNsRules] Running namespace: %s query...", namespace);
                const results = registry.query(query);
                this._logger.info("[_loadNsRules] count: %s", results.length);
                for (const result of results) {
                    this._loadNsRule(result);
                }
            }
        }
        else {
            const query = {
                isApiVersion: false,
                apiOrNone: KUBEVIOUS_API_NAME,
                kind: KUBEVIOUS_KIND_RULE,
                isAllNamespaces: true,
            };
            this._logger.info("[_loadNsRules] Running global query...");
            const results = registry.query(query);
            this._logger.info("[_loadNsRules] count: %s", results.length);
            for (const result of results) {
                this._loadNsRule(result);
            }
        }
        spinner.complete("Rules loaded.");
    }
    _loadNsRule(manifest) {
        var _a, _b, _c, _d, _e, _f;
        const config = manifest.config;
        const name = (_a = config.metadata) === null || _a === void 0 ? void 0 : _a.name;
        const namespace = (_c = (_b = config.metadata) === null || _b === void 0 ? void 0 : _b.namespace) !== null && _c !== void 0 ? _c : 'default';
        this._logger.info("[_loadNsRule] %s :: %s...", namespace, name);
        const spec = (_d = config.spec) !== null && _d !== void 0 ? _d : {};
        spec.disabled = (_e = spec.disabled) !== null && _e !== void 0 ? _e : false;
        spec.values = (_f = spec.values) !== null && _f !== void 0 ? _f : {};
        const targetScript = spec.target;
        const ruleScript = spec.rule;
        if (!name) {
            return;
        }
        this._rules[manifest.idKey] = {
            manifest: manifest,
            source: manifest.source.source,
            kind: types_1.RuleKind.Rule,
            namespace: namespace,
            name: name,
            application: {
                namespace: namespace
            },
            target: targetScript,
            script: ruleScript,
            values: spec.values,
            spec: spec
        };
    }
    _loadApplicatorRules(registry, options) {
        const spinner = (0, spinner_1.spinOperation)("Loading RuleApplicators...", 2);
        if (options.onlySelectedNamespaces) {
            for (const namespace of options.namespaces) {
                spinner.update(`Loading RuleApplicators from ${namespace}...`);
                const query = {
                    isApiVersion: false,
                    apiOrNone: KUBEVIOUS_API_NAME,
                    kind: KUBEVIOUS_KIND_APPLICATOR_RULE,
                    namespace: namespace
                };
                const results = registry.query(query);
                for (const result of results) {
                    this._loadApplicatorRule(result);
                }
            }
        }
        else {
            const query = {
                isApiVersion: false,
                apiOrNone: KUBEVIOUS_API_NAME,
                kind: KUBEVIOUS_KIND_APPLICATOR_RULE,
                isAllNamespaces: true,
            };
            const results = registry.query(query);
            for (const result of results) {
                this._loadApplicatorRule(result);
            }
        }
        spinner.complete("RuleApplicators loaded");
    }
    _loadApplicatorRule(manifest) {
        var _a, _b, _c, _d, _e, _f;
        const config = manifest.config;
        const name = (_a = config.metadata) === null || _a === void 0 ? void 0 : _a.name;
        const namespace = (_c = (_b = config.metadata) === null || _b === void 0 ? void 0 : _b.namespace) !== null && _c !== void 0 ? _c : 'default';
        this._logger.info("[_loadApplicatorRule] %s :: %s...", namespace, name);
        const spec = (_d = config.spec) !== null && _d !== void 0 ? _d : {};
        spec.disabled = (_e = spec.disabled) !== null && _e !== void 0 ? _e : false;
        spec.values = (_f = spec.values) !== null && _f !== void 0 ? _f : {};
        if (!name) {
            return;
        }
        this._ruleApplicators[manifest.idKey] = {
            manifest: manifest,
            source: manifest.source.source,
            kind: types_1.RuleKind.RuleApplicator,
            namespace: namespace,
            name: name,
            application: {
                namespace: namespace
            },
            values: spec.values,
            spec: spec
        };
    }
}
exports.RuleRegistry = RuleRegistry;
//# sourceMappingURL=rule-registry.js.map